AJS.$(document).ready(function() {

    VM$.configuration.getShowUserPickerVacationWarning(function(doShow) {
        if (doShow) {
            var userVacationInfo = new UserVacationInfo();

            findUserPickerFields(userVacationInfo);
            findUserFields(userVacationInfo);
            JIRA.bind(JIRA.Events.NEW_CONTENT_ADDED, function(e, context, reason) {
                if (reason == 'dialogContentReady' || reason == 'dialogReady') {
                    findUserPickerFields(userVacationInfo);
                    findUserFields(userVacationInfo);
                }
            });

            JIRA.bind(JIRA.Events.PANEL_REFRESHED, function(e, context, reason) {
                findUserPickerFields(userVacationInfo);
                findUserFields(userVacationInfo);
            });

            JIRA.bind(JIRA.Events.ISSUE_REFRESHED, function(e, context, reason) {
                findUserPickerFields(userVacationInfo);
                findUserFields(userVacationInfo);
            });
        }
    });
});

function UserVacationInfo() {
    var ajaxCacher = new AjaxCacher(AJS.$);

    function getCurrentVacationInfoForUser(userName, responseCallback) {

        ajaxCacher.ajax({
            type : "GET",
            url : AJS.contextPath() + "/rest/vacation-manager/1.0/user-vacation/is-on-vacation/user/" + userName,
            contentType : 'application/json; charset=utf-8',
            async : true,
            cacheId : userName
        }, responseCallback);
    }

    return {
        getCurrentVacationInfoForUser : getCurrentVacationInfoForUser
    }
}

function findUserPickerFields(userVacationInfo) {
    AJS.$("input.userpickerfield,select.single-user-picker").each(function() {
        var obj = AJS.$(this);
        if (obj.attr("vm-inited") != "true") {
            obj.attr("vm-inited", "true");
            try {
                new UserPickerWarning(obj, userVacationInfo);
            } catch (ex) {
                console.log("Cannot parse user picker field: " + ex);
            }
        }
    });
}

function UserPickerWarning(field, userVacationInfo) {
    var messageContainer = null;

    function initMessageContainer() {
        messageContainer = $("<div class='vm-warning-field-group'></div>");
        field.closest("div.field-group").append(messageContainer);
        messageContainer.hide();
    }
    
    field.on('remove', hideWarningMessage);

    field.on('change', function() {
        checkUser(getUserName());
    });

    function checkUser(userName) {
    	if (!userName) {
    		hideWarningMessage();
    		return;
    	}

        userVacationInfo.getCurrentVacationInfoForUser(userName, function(data) {
            if (data && data.hasVacation) {
                showWarningMessage(data.message);
            } else {
                hideWarningMessage();
            }
        });
    }
    
    function hideWarningMessage() {
        messageContainer.hide();
    }

    function showWarningMessage(message) {
        messageContainer.html(message);
        messageContainer.show();
    }

    function getUserName() {
        return field.val().toString();
    }

    initMessageContainer();
    checkUser(getUserName());
}

function findUserFields(userVacationInfo) {
    AJS.$("#assignee-val,#reporter-val,[data-fieldtype='userpicker']").each(function() {
        var obj = AJS.$(this);
        if (obj.attr("vm-inited") != "true") {
            obj.attr("vm-inited", "true");
            try {
                new UserFieldWarning(obj, userVacationInfo);
            } catch (ex) {
                console.log("Cannot parse user field: " + ex);
            }
        }
    });
}

function UserFieldWarning(field, userVacationInfo) {
    var userName = field.attr("rel");
    if (userName == undefined) {
        userName = field.find("[rel]").attr("rel");
    }
    
    if (!userName) {
    	return;
    }

    userVacationInfo.getCurrentVacationInfoForUser(userName, function(data) {
        if (data && data.hasVacation) {
            addWarningMessage(data.substitute, data.message);
        }
    });

    function addWarningMessage(substitute, message) {
        var messageContainer = $("<div class='vm-warning-field-group'></div>");
        messageContainer.html("<div>"+message+"</div>");
        field.after(messageContainer);
    }
}
